#!/usr/bin/python3
import os
import sys
import gi
import json
from threading import Thread
from select import select

gi.require_version('Gtk', '3.0')
gi.require_version('AppIndicator3', '0.1')

from gi.repository import Gtk as gtk, AppIndicator3 as appindicator
from gi.repository import GLib

reading = True

Dirname = os.path.dirname(os.path.realpath(__file__))
NormalIcon = os.path.join(Dirname, 'tray/linux/tray.png')
SnoozeIcon = os.path.join(Dirname, 'tray/linux/tray_s.png')
UnreadIcon = os.path.join(Dirname, 'tray/linux/tray_dot.png')

class ReaderThread(Thread):
    def __init__(self, indicator):
        Thread.__init__(self)
        self.indicator = indicator

    def run(self):
        global reading

        while reading:
            rlist, wlist, xlist = select([sys.stdin], [], [], 1)
            if rlist:
                for line in rlist[0]:
                    line = line.rstrip('\n')
                    # https://pygobject.readthedocs.io/en/latest/guide/threading.html
                    GLib.idle_add(self.handle_request, line)

    def handle_request(self, line):
        req = json.loads(line)

        if req['api'] == 'updateTooltip':
            self.indicator.set_title(req['params'])
        elif req['api'] == 'updateImg':
            if req['params'] == 'normal':
                self.indicator.set_icon(NormalIcon)
            elif req['params'] == 'snooze':
                self.indicator.set_icon(SnoozeIcon)
            elif req['params'] == 'unread':
                self.indicator.set_icon(UnreadIcon)
            else:
                print('incorrect icon', file=sys.stderr, flush=True)

class Tray():
    def __init__(self, stringJson, indicator):
        self._t = json.loads(stringJson)

        self.indicator = indicator
        self.indicator.set_status(appindicator.IndicatorStatus.ACTIVE)
        self.indicator.set_menu(self.menu())

        gtk.main()

    def menu(self):
        menu = gtk.Menu()

        openMenuItem = gtk.MenuItem(self._t['menu']['openwindow'])
        openMenuItem.connect('activate', lambda _: self.send_request('openWindow'))

        snoozeMenuItem = gtk.MenuItem(self._t['menu']['snooze'])
        snoozeMenuItem.set_submenu(self.submenu())

        quitMenuItem = gtk.MenuItem(self._t['menu']['quit'])
        quitMenuItem.connect('activate', self.quit)

        menu.append(openMenuItem)
        menu.append(snoozeMenuItem)
        menu.append(quitMenuItem)

        menu.show_all()

        return menu

    def submenu(self):
        submenu = gtk.Menu()

        item1 = gtk.MenuItem('20 {}'.format(self._t['common']['minutes']))
        item2 = gtk.MenuItem('1 {}'.format(self._t['common']['hour']))
        item3 = gtk.MenuItem('4 {}'.format(self._t['common']['hours']))

        item1.connect('activate', lambda _: self.send_request('snooze', 20))
        item2.connect('activate', lambda _: self.send_request('snooze', 60))
        item3.connect('activate', lambda _: self.send_request('snooze', 240))

        submenu.append(item1)
        submenu.append(item2)
        submenu.append(item3)

        return submenu

    def send_request(self, api, params=None):

        req = {
            'api': api,
            'params': params
        }

        ret = '<req>{}</req>'.format(json.dumps(req))

        print(ret, flush=True)

    def quit(self, _):
        global reading
        reading = False

        self.send_request('quit')

        sys.stdin.close()
        gtk.main_quit()

if __name__ == "__main__":
    lang = sys.stdin.readline()

    # https://gjs-docs.gnome.org/appindicator301~0.1_api/
    # https://gjs-docs.gnome.org/gtk30~3.24.29/gtk.menu
    indicator = appindicator.Indicator.new('chat', NormalIcon, appindicator.IndicatorCategory.APPLICATION_STATUS)

    thread = ReaderThread(indicator)
    thread.start()

    tray = Tray(lang, indicator)

    thread.join()

